/*
 React Studio wrapper for the 'react-trend' npm package.

 - 2017.06.12 / Pauli Ojala / pauli@neonto.com

 --- License: public domain (created by Neonto Ltd; you may modify and reuse this code freely)
 */


// -- plugin info requested by host app --

this.describePlugin = function(id, lang) {
  switch (id) {
    case 'displayName':
      return "Trend Graph";

    case 'shortDisplayText':
      return "Visualize data as a trend line.";

    case 'defaultNameForNewInstance':
      return "trend";
  }
}


// -- private variables --

this._data = {
  linkedDataSheet: "",
  linkedColumnName: "value",
  smooth: true,
  autoDraw: true,
  autoDrawDuration: 2,
  autoDrawEasing: 'ease-out',
  radius: 10,
  strokeWidth: 2,
  strokeLinecap: 0,  // UI multibutton item values are 'butt', 'round', 'square'
	baseColor: [0.5, 0.9, 1, 1],
  useGradientColor2: true,
  gradientColor2: [0.9, 0.9, 0.3, 1],
  useGradientColor3: false,
  gradientColor3: [1.0, 0.7, 0.2, 1],
};


// -- persistence, i.e. saving and loading --

this.persist = function() {
  return this._data;
}

this.unpersist = function(data) {
	this._data = data;
}


// -- inspector UI --

this.inspectorUIDefinition = [
  {
    "type": "label",
    "text": "Choose a data sheet that provides the data to display:"
  },
  {
    "type": "datasheet-picker",
    "id": "linkedDataSheet",
    "actionBinding": "this._onUIChange"
  },
  {
    "type": "label",
    "text": "Column name in data sheet:"
  },

  {
    "type": "textinput",
    "id": "linkedColumnName",
    "actionBinding": "this._onUIChange",
  },

  {
    "paddingTop": 20,
    "type": "label",
    "text": "Following settings affect the graph's look:"
  },  
  {
    "paddingTop": 8,
    "type": "checkbox",
    "id": "smooth",
    "label": "Smoothed curve",
    "actionBinding": "this._onUIChange"
  },
  {
    "type": "checkbox",
    "id": "autoDraw",
    "label": "Auto-draw (animated line drawing)",
    "actionBinding": "this._onUIChange"
  },
  {
    "type": "numberinput",
    "id": "autoDrawDuration",
    "label": "Auto-draw duration",
    "min": 0,
    "max": 5,
    "increment": 0.1,
    "actionBinding": "this._onUIChange"
  },
  {
    "type": "numberinput",
    "id": "radius",
    "label": "Radius",
    "min": 1,
    "max": 20,
    "increment": 1,
    "actionBinding": "this._onUIChange"
  },

  {
    "paddingTop": 20,
    "type": "label",
    "text": "Line style:"
  },  
  {
    "type": "multibutton",
    "id": "strokeLinecap",
    "actionBinding": "this._onUIChange",
    "labels": ["butt", "round", "square"]
  },
  {
    "type": "numberinput",
    "id": "strokeWidth",
    "label": "Stroke width",
    "min": 1,
    "max": 20,
    "increment": 1,
    "actionBinding": "this._onUIChange"
  },


  {
    "paddingTop": 20,
    "type": "label",
    "text": "Colors:"
  },    
  {
    "type": "color-picker",
    "id": "baseColor",
    "actionBinding": "this._onUIChange",
    "label": "Base color"
  },
  
  {
    "type": "checkbox",
    "paddingTop": 6,
    "id": "useGradientColor2",
    "label": "Enable gradient color 2",
    "actionBinding": "this._onUIChange"
  },
  {
    "type": "color-picker",
    "id": "gradientColor2",
    "actionBinding": "this._onUIChange",
    "label": "Color 2"
  },
  {
    "type": "checkbox",
    "paddingTop": 6,
    "id": "useGradientColor3",
    "label": "Enable gradient color 3",
    "actionBinding": "this._onUIChange"
  },
  {
    "type": "color-picker",
    "id": "gradientColor3",
    "actionBinding": "this._onUIChange",
    "label": "Color 3"
  },
  
];

this._accessorForDataKey = function(key) {
  // This method provides a convenient way to map controls declared in "inspectorUIDefinition" to properties specific for each control type.
  // Both onCreateUI and onUIChange will call this method.
  // It is assumed here that your _data object will store values with the same property names as each controlId.
  // You don't have to use this for your own UIs, but it's a convenient way to get two-way binding between the UI and your _data object.
  //
  var accessorsByControlType = {
    'textinput': 'text',
    'checkbox': 'checked',
    'numberinput': 'numberValue',
    'multibutton': 'numberValue',
    'color-picker': 'rgbaArrayValue',
    'element-picker': 'elementId',
    'screen-picker': 'screenName',
    'dataslot-picker': 'dataSlotName',
    'datasheet-picker': 'dataSheetName'
  }
  var accessorsByControlId = {};
  for (var control of this.inspectorUIDefinition) {
    var prop = accessorsByControlType[control.type];
    if (prop && control.id)
      accessorsByControlId[control.id] = prop;
  }
  return accessorsByControlId[key];
}

this.onCreateUI = function() {
  // Bind values in private _data to UI automatically using "_accessorForDataKey" above.
  var ui = this.getUI();
  for (var controlId in this._data) {
    var prop = this._accessorForDataKey(controlId);
    if (prop) ui.getChildById(controlId)[prop] = this._data[controlId];
  }
}

this._onUIChange = function(controlId) {
  // This is the "actionBinding" specified for controls where we want to use the automatic binding to _data.
  var ui = this.getUI();
  var prop = this._accessorForDataKey(controlId);
  if (prop) {
    console.log("updated: "+controlId);
    this._data[controlId] = ui.getChildById(controlId)[prop];
  } else {
    console.log("** no data property found for controlId "+controlId);
  }
}


// -- plugin preview --

this.renderIcon = function(canvas) {
  var ctx = canvas.getContext('2d');
  var w = canvas.width;
  var h = canvas.height;
  ctx.save();
  
  if (this.icon == null) {
    var path = Plugin.getPathForResource("plugin_logo.png");
    this.icon = Plugin.loadImage(path);
  }
  if (this.icon) {
    var iconW = this.icon.width;
    var iconH = this.icon.height;
    var aspectScale = Math.min(w/iconW, h/iconH);
    var scale = 0.72 * aspectScale; // add some margin around icon
    ctx.globalAlpha = 0.97;
    ctx.drawImage(this.icon, (w-iconW)/2, (h-iconH)/2, iconW, iconH);    
  }
  
  ctx.restore();
};

this.renderEditingCanvasPreview = function(canvas, controller) {
  var ctx = canvas.getContext('2d');
  var w = canvas.width;
  var h = canvas.height;
  ctx.save();
  
  var displayScale = 1;
  if (controller && (displayScale = controller.renderPixelsPerWebPixel)) {
    ctx.scale(displayScale, displayScale);
    w /= displayScale;
    h /= displayScale;
  }
  
  if ( !this.previewImage) {
    this.previewImage = Plugin.loadImage(Plugin.getPathForResource("plugin_screenshot@2x.png"));
  }
	
  if (this.previewImage) {
    var img = this.previewImage;
    var imgW = img.width;
    var imgH = img.height;
    var dstAsp = w / h;

    // The 'react-trend' component maintains a 4:1 aspect ratio.
    // If the canvas is wider, fit into 100% height.
    if (dstAsp >= 4) {
      var dstW = (h / imgH) * imgW;
      var dstH = h;
      ctx.drawImage(img, (w - dstW)/2, 0, dstW, dstH);
    }
    else {
      var dstW = w;
      var dstH = (w / imgW) * imgH;
      ctx.drawImage(img, 0, (h - dstH)/2, dstW, dstH);
    }
  }
  
  ctx.font = 'italic 12px Helvetica';
  ctx.fillStyle = 'rgba(0, 0, 0, 0.2)';
  var textY = w/8 + w/16;
  ctx.fillText("PREVIEW", 10, textY);
  ctx.fillText("Doesn't update dynamically", 10, textY + 16);

  
  ctx.restore();
}


// -- layout UI --

this.defaultContentSizeInWebPixels = [300, 75];

this.hasFixedContentAspectRatio = true;



// -- code generation, React web --

this.getReactWebPackages = function() {
  // Return dependencies that need to be included in the exported project's package.json file.
  // Each key is an npm package name that must be imported, and the value is the package version.
  // 
  // Example:
  //    return { "somepackage": "^1.2.3" }
  
  return {
    "react-trend": "^1.2.4"
  };
}

this.getReactWebImports = function(exporter) {
	var arr = [
    { varName: "Trend", path: "react-trend" }
  ];
	
	return arr;
}

this.writesCustomReactWebComponent = false;


this.getReactWebRenderMethodSetupCode = function(exporter, elementName) {
  var dataSheetCode = exporter.valueAccessForDataSheetByName(this._data.linkedDataSheet);
  if (dataSheetCode) {
    this._reactRenderDataVarName = `data_${elementName}`;
    
    return ""
    +`const sheet_${elementName} = ${dataSheetCode};\n`
    +`const ${this._reactRenderDataVarName} = sheet_${elementName}.items.map((item) => {\n`
    +`  return parseFloat(item['${this._data.linkedColumnName}']);\n`
    +`});\n`
  }
}

this.getReactWebJSXCode = function(exporter) {
  var jsx = `<Trend`;

  if (this._reactRenderDataVarName) {
    jsx += ` data={${this._reactRenderDataVarName}}`;
  }
    
  if (this._data.smooth) {
    jsx += " smooth";
  }
  if (this._data.autoDraw) {
    jsx += ` autoDraw autoDrawDuration={${(parseFloat(this._data.autoDrawDuration)*1000).toFixed(0)}}`;
  }
  
  jsx += ` radius={${this._data.radius}}`;
  jsx += ` strokeWidth={${this._data.strokeWidth}}`;
  
  // Line cap style is stored as an integer enum
  var linecap = parseInt(this._data.strokeLinecap);
  var linecapStr = (linecap === 0) ? 'butt' : (linecap === 1) ? 'round' : 'square';
  jsx += ` strokeLinecap={'${linecapStr}'}`;
  
  // The 'react-trend' interface only takes a gradient with at least two colors,
  // so if there's a single color specified we'll just use a gradient with the same color twice.
  var color1 = this._rgbaFromColorArray(this._data.baseColor);
  var color2 = color1;
  var color3 = null;
  if (this._data.useGradientColor2) {
    color2 = this._rgbaFromColorArray(this._data.gradientColor2);
  }
  if (this._data.useGradientColor3) {
    color3 = this._rgbaFromColorArray(this._data.gradientColor3);
  }
  var gradientStr = (color3) ? `['${color1}', '${color2}', '${color3}']` : `['${color1}', '${color2}']`;
  jsx += ` gradient={${gradientStr}}`;
		
  jsx += ` />`;
  return jsx;
}

this.getReactWebCSS = function(exporter, elementBaseSelector, screenFormatId) {
  var css = "";
  
  css += ""
  + `${elementBaseSelector} svg {\n`
  + `  width: 100%;\n`
  + `  height: 100%;\n`
  + `}\n`;
  
  return css;
}

// utility function to write HTML colors
this._rgbaFromColorArray = function(c) {
  return `rgba(${255*c[0]}, ${255*c[1]}, ${255*c[2]}, ${c[3]})`;
}



